
import csv
from io import StringIO

def psql_insert_copy(table, conn, keys, data_iter):
    """
    Execute SQL statement inserting data

    Parameters
    ----------
    table : pandas.io.sql.SQLTable
    conn : sqlalchemy.engine.Engine or sqlalchemy.engine.Connection
    keys : list of str
        Column names
    data_iter : Iterable that iterates the values to be inserted
    """
    # gets a DBAPI connection that can provide a cursor
    dbapi_conn = conn.connection
    with dbapi_conn.cursor() as cur:
        s_buf = StringIO()
        writer = csv.writer(s_buf)
        writer.writerows(data_iter)
        s_buf.seek(0)

        columns = ', '.join(['"{}"'.format(k) for k in keys])
        if table.schema:
            table_name = '{}.{}'.format(table.schema, table.name)
        else:
            table_name = table.name

        sql = 'COPY {} ({}) FROM STDIN WITH CSV'.format(
            table_name, columns)
        cur.copy_expert(sql=sql, file=s_buf)

def to_sql(dataframe, name, postgresql_con, *, schema=None, if_exists='fail', index=True, index_label=None, chunksize=None, dtype=None):
    """
    Write records stored in a DataFrame to a PostgreSQL database.
    """
    dataframe.to_sql(
        name=name,
        con=postgresql_con,
        schema=schema,
        if_exists=if_exists,
        index=index,
        index_label=index_label,
        chunksize=chunksize,
        dtype=dtype,
        method=psql_insert_copy)



from setuptools import setup, find_packages

setup(
    name="bangpg",
    version="0.1.1",
    author="MATHEUS DE SA DE SOUZA",
    author_email="matheusdesa55@gmail.com",
    description="This packeges implements a faster alternative `to_sql()` method for PostgreSQL that support COPY FROM.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/matheus0sa/bangpg",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)